//-----------------------------------------
// LScript Generic LightMesh
//
// Copyright 2006 Michael Green
// info@shortbutpurple.fsnet.co.uk
//

myversion=0.1;
@version 2.2
@warnings
@script generic

// global values go here

generic
{
    mymaps[1]="<none>";
    mywmaps[1]="<none>";
    mymap=VMap();
    i=1;
    j=2;
    while(mymap)
    {
        if(mymap.type==VMRGB || mymap.type==VMRGBA)
        {
            mymaps[i]=mymap.name;
            i++;
        }
        if(mymap.type==VMWEIGHT)
        {
            mywmaps[j]=mymap.name;
            j++;
        }
        mymap=mymap.next();
    }
    if(mymaps[1]=="<none>")
    {   
        error("No Colour Vmaps found");
        return;
    }
    
    mymeshname=recall("MGLightMeshMesh",nil);
    my0int=recall("MGLightMesh0int",0);
    my100int=recall("MGLightMesh100int",1.0);
    mylightname=recall("MGLightMeshLight",nil);
    mydir=recall("MGLightMeshDir",1);
    mytargetname=recall("MGLightMeshTarget",nil);

    if(mymeshname)
        mymesh=Mesh(mymeshname);
    if(mylightname)
        mylight=Light(mylightname);
    if(mytargetname)
        mytgt=Mesh(mytargetname);

    // perform all processing here

    reqbegin("MG LightMesh Ver:"+myversion);

    c0 = ctlmeshitems("Lighting Mesh",mymesh);
    c1 = ctlpopup("Vertex Colour Map",1,mymaps);
    c2 = ctlpopup("Weightmap Intensity Control",1,mywmaps);
    c3 = ctlpercent("Intensity at 0% weight",my0int);
    c4 = ctlpercent("Intensity at 100% weight",my100int);
    c5 = ctllightitems("Light Prototype",mylight);
//    c6 = ctlchoice("Light Direction",mydir,@"Target","Normal","Flipped Normal"@);
    c7 = ctlallitems("Light Target",mytgt);
    c8 = ctlpopup("Help",1,@"This is a tool for generating coloured light domes",
                            "from a vertex coloured mesh. You could colour the mesh",
                            "yourself, or use the LW surface baker to bake an image's",
                            "colour to a vertex map, then use that to make the dome.",
                            "Lighting mesh is a shape at each vertex of which",
                            "will be placed a copy of the Light Prototype.",
                            "The colour of the light will be set to that vertex's",
                            "colour as in the chosen Vertex Colour map.",
                            "The intensity of the light can be controlled with the",
                            "weightmap in conjunction with the 0% and 100%",
                            "settings. Your weightmap can go above/below this range."@);
//                            "The direction of the light is either pointing at a ",
//                            "designated target item, or along the point's normal ",
//                            "(so flip the object's normal if they all end up ",
//                            "facing outwards)."@);
    return if !reqpost();

    // get requester control values here
    mymesh=getvalue(c0);
    mycmapname=getvalue(c1);
    mywmapname=getvalue(c2);
    my0int=getvalue(c3);
    my100int=getvalue(c4);
    mylight=getvalue(c5);
//    mydir=getvalue(c6);
    mydir=1;
    mytgt=getvalue(c7);

    reqend();

//debug();
    if(!mymesh||mymesh.null)
    {
        error("Please select the lighting mesh");
        return;
    }

    if(!mylight)
    {
        error("Please select the light prototype");
        return;
    }

    mycmap=VMap(mymaps[mycmapname]);

    if(!mycmap)
    {
        error("Please select the lighting colourmap");
        return;
    }

    if(mydir==1 && mytgt==nil)
    {
        error("Please select a target for the lights");
        return;
    }

    mywmap=nil;
    if(myweightmapname)
        mywmap=VMap(mywmaps[mywmapname]);
    mesh2lights(mymesh,mycmap,mywmap,my0int,my100int,mylight,mydir,mytgt);

    store("MGLightMeshMesh",mymesh.name);
    store("MGLightMesh0int",my0int);
    store("MGLightMesh100int",my100int);
    store("MGLightMeshLight",mylight.name);
    store("MGLightMeshDir",mydir);
    store("MGLightMeshTarget",mytgt.name);

    info("Done");
}

mesh2lights:pmesh,pcmap,pwmap,p0int,p100int,plight,pdir,ptgt
{
//debug();
    if(!pmesh||pmesh.null||!plight||!pcmap||(pdir==1&&ptgt==nil))
    {
        return;
    }

    if(pmesh.pointCount()<3||pmesh.polygonCount()<1)
        return;

    if(pdir- 1)
    {   
        mynormals=FindPointNormals(pmesh);
    }
//    mypnts=pmesh.points;
//    debug();
    SelectItem(plight.id);
    for (i=1;i<=pmesh.pointCount();i++)
    {
        mypoint=pmesh.points[i];
        pos=pmesh.position(mypoint);
        Clone(1);

// do colour
        mycolour=@1,1,1@;
        if(pcmap.isMapped(mypoint))
            mycolour=pcmap.getValue(mypoint);
        LightColor(mycolour[1],mycolour[2],mycolour[3]);

// do intensity
        mytempint[1]=1.0;
        if(pwmap && pwmap.isMapped(mypoint))
            mytempint=pwmap.getValue(mypoint);
        myint=(p100int-p0int)*mytempint[1]+p0int;
//        myint=min(myint,p100int);
//        myint=max(myint,p0int);

        LightIntensity(myint);

//do position
        Position(pos.x,pos.y,pos.z);
        if(pdir - 1)
        {
            Rotation(mynormal.x,mynormal.y,mynormal.z);
        }
        else
        {
            TargetItem(ptgt.id);
            HController(2);
            PController(2);
        }
        ParentItem(pmesh.id);

        RefreshNow();

    }
    AutoConfirm(1);
    SelectItem(plight.id);
    ClearSelected();
    AutoConfirm(0);

}

FindPointNormals:pmesh
{
//        find surface normals
        //find lowest pointid
debug();
        mylowpoint=pmesh.points[1];
        myparray[1]=1;
//        for (i=2;i<=pmesh.pointCount();i++)
//        {
//            mypoint=pmesh.points[i];
//             mylowpoint=min(mypoint,mylowpoint);
//        }
        //place poly normals in an array based on pointid

        for (i=1;i<=pmesh.polygonCount();i++)
        {
            mypoly=pmesh.polygons[i];
            if(pmesh.vertexCount(mypoly)>=3)
            {
                for(j=1;j<=pmesh.vertexCount(mypoly)||j<=3;j++)
                {
                    myvert=pmesh.vertex(mypoly,j);
                    mypoints[j]=pmesh.position(myvert);
                }

                veca=mypoints[2]-mypoints[1];
                vecb=mypoints[3]-mypoints[2];
                mynormal=cross3d(veca,vecb);
                for(j=1;j<=pmesh.vertexCount(mypoly)||j<3;j++)
                {
                    index=pmesh.vertex(mypoly,j);//-mylowpoint;
                    if(myparray[index])
                        myparray[index][myparry.size()+1]=mynormal;
                    else
                        myparray[index][1]=mynormal;
                }
            }
        }

        for(i=1;i<=myparray.size();i++)
        {
            if(myparray[i])
            {
                total=<0,0,0>;
                for(j=1;j<=myparray[i].size();j++)
                {
                    total=total+myparray[i][j];
                }
                total=total/j;
                myrotation[1]=atan(total.x/total.z);
                myrotation[1]=atan(total.y/total.z);
                myrotation[3]=0;
                mynewarray[i]=myrotation;
            }
        }
        return(mynewarray);
}

AverageNormals:ppolys
{
    myvec=<0,0,0>;
    for(j=1;j<=mypolys.size();j++)
    {
        mypoly=mypolys[j];
        if(mypoly.pointCount()>3 && !mypoly.isCurve())
        {
            sidevec1=mypoly.points[2]-mypoly.points[1];
            sidevec2=mypoly.points[3]-mypoly.points[2];
            polynorm=cross3d(sidevec1,sidevec2);
            myvec=myvec+polynorm;
        }
    }
    myvec=myvec/mypolys.size();
    return(myvec);
}